#ifndef _PROFILE_H
#define _PROFILE_H

#include "timing.h"

struct thread_ctx;
enum thread_selector;

struct prof_event_header {
	unsigned char	type;
};

struct prof_start_end_data {
	unsigned long long	start;
	unsigned long long	end;
	unsigned long long	_unused;
};

struct prof_completion_data {
	unsigned long long	total_time;
	unsigned long long	frames_decoded;
	unsigned int		pid_group;
	unsigned int		_unused;
};

struct prof_jitter_data {
	unsigned long long	frame;
	long long		jitter;
	unsigned long long	time;
};

struct prof_sem_wait_data {
	unsigned long long	frame;
	unsigned long long	_unused[2];
};

struct prof_buffer_data {
	unsigned long long	frame;
	unsigned long long	time;
	unsigned int		queued;
	unsigned int		_unused;
};

struct prof_overflow_data {
	unsigned long long	_unused[3];
};


/* starts at one so we can spot uninitialized records */
typedef enum {
	PROF_READ_FRAME = 1,	/* av_read_frame() */
	PROF_DECODE_VIDEO,	/* avcodec_decode_video2() */
	PROF_SWS_SCALE,		/* sws_scale in draw_frame */
	PROF_DRAW_FRAME,	/* frame drawing with SDL */
	PROF_COMPLETION,	/* thread exit, record total time */
	PROF_FULL_DECODE,	/* time to decode, start to end */
	PROF_FULL_DISPLAY,	/* time to display, start to end */
	PROF_JITTER_ABS,	/* time lagging behind (or ahead) */
	PROF_SEM_WAIT,		/* LITMUS had to wait on a semaphore */
	PROF_JITTER_RELATIVE,	/* relative jitter (to last measurement) */
	PROF_JITTER_RELEASE,	/* release jitter */
	PROF_PICT_COPY,		/* copying picture to screen */
	PROF_RESPONSE,		/* response time */
	PROF_BUFFER,		/* buffer info */
	PROF_OVERFLOW,		/* overflowed error */
} prof_record_type_t;


#define DATA(x) struct prof_ ## x ## _data x;

struct prof_event_record {
	struct prof_event_header hdr;
	union {
		DATA(start_end);
		DATA(completion);
		DATA(jitter);
		DATA(sem_wait);
		DATA(buffer);
		DATA(overflow);
	} data;
};

/*
 * Various function prototypes
 */

int next_record_idx(struct thread_ctx*, enum thread_selector);

void prof_completion_event(struct thread_ctx*,
		const enum thread_selector, const unsigned long long,
		const unsigned long long);

void __jitter_rec(struct thread_ctx*, const enum thread_selector,
		const prof_record_type_t, const unsigned long long,
		const long long, const unsigned long long);

void prof_jitter_rec(struct thread_ctx*, const enum thread_selector,
		const prof_record_type_t, const unsigned long long,
		const struct timespec*, const mono_time_t*);

void prof_buffer_rec(struct thread_ctx*, const enum thread_selector,
		const unsigned long long, const int);

#ifdef USE_PROFILE

void prof_start_end(struct thread_ctx*, const enum thread_selector,
		prof_record_type_t, unsigned long long, unsigned long long);

void prof_sem_wait(struct thread_ctx*, const enum thread_selector,
		unsigned long long);

#define PROFILE_FUNCTION_CALL(t_ctx, thread, type, ret, funct,		\
		funct_args...)						\
	do {								\
		unsigned long long start = cpu_time();			\
		ret = funct(funct_args);				\
		unsigned long long end = cpu_time();			\
		prof_start_end(t_ctx, thread, type, start, end);	\
	} while (0)

#define PROFILE_VOID_FUNCTION_CALL(t_ctx, thread, type, funct,		\
		funct_args...)						\
	do {								\
		unsigned long long start = cpu_time();			\
		funct(funct_args);					\
		unsigned long long end = cpu_time();			\
		prof_start_end(t_ctx, thread, type, start, end);	\
	} while (0)

#else /* from if USE_PROFILE */

/*
 * Writing (void) variable_name; lets us not use that variable, but avoid the
 * "unused variable" warning from GCC when -Wall is on
 */
#define prof_start_end(t_ctx, thread, rec_type, start, end)		\
	do {								\
		(void) start;						\
		(void) end;						\
	} while (0)

#define prof_sem_wait(t_ctx, thread, frame)				\
	do {								\
	} while (0)

#define PROFILE_FUNCTION_CALL(t_ctx, thread, type, ret, funct,		\
		funct_args...)						\
	do {								\
		ret = funct(funct_args);				\
	} while (0)

#define PROFILE_VOID_FUNCTION_CALL(t_ctx, thread, type, funct,		\
		funct_args...)						\
	do {								\
		funct(funct_args);					\
	} while (0)

#endif /* from if USE_PROFILE */

#endif
