#ifndef _SEM_COMMON_H
#define _SEM_COMMON_H

#include <semaphore.h>

#ifdef USE_LITMUS
#include <errno.h>
#include <stdio.h>
#endif

#include "profile.h"
#include "common.h"

struct sems {
	sem_t *vidq_full;
	sem_t *vidq_empty;

	char *vidq_name_full;
	char *vidq_name_empty;
};

struct thread_ctx;

void initialize_sems(struct thread_ctx*);
sem_t *sem_new(const char*, int);
void destroy_sems(struct sems*);

static inline void sem_post_wrap(sem_t *sem)
{
	if (sem_post(sem))
		die("Error on sem_post\n");
}

static inline void __sem_wait(sem_t *sem)
{
	errno = 0;
	if (0 != sem_wait(sem))
	{
		if (errno == EINTR && should_quit())
			fprintf(stderr, "semaphore interrupted by quit\n");
		else
			die("Error on sem_wait");
	}
}

static inline void sem_wait_wrap(sem_t *sem, struct thread_ctx *t_ctx,
		const enum thread_selector thread,
		unsigned long long frame)
{
#ifdef USE_LITMUS
	/* LITMUS-case: We rarely need to use semaphores */
	errno = 0;
	if (0 != sem_trywait(sem))
	{
		if (errno == EAGAIN)
		{
			/* record we had to wait, then really do it */
			prof_sem_wait(t_ctx, thread, frame);
			__sem_wait(sem);
		}
		else if (!(errno == EINTR && should_quit()))
		{
			die("Error on sem_trywait");
		}
	}
#else
	/* Non-LITMUS, we expect to wait */
	__sem_wait(sem);
#endif
}

#endif
