#ifdef USE_LITMUS

#include <stdio.h>

#include "litmus.h"

#include "inc/common.h"
#include "inc/options.h"
#include "inc/thread_ctx.h"

#define NSEC_PER_MSEC 1000000

#define CALL( exp ) \
	do { \
		int ret; \
		ret = exp; \
		if (ret != 0) \
			die(#exp " failed\n"); \
		else \
			printf(#exp " OK\n"); \
	} while (0)

static void make_litmus_task(lt_t exec, lt_t period, lt_t phase, int cpu,
		task_class_t cls, budget_policy_t budget_policy,
		int set_cpu_set)
{
	CALL( init_litmus() );
	CALL( sporadic_task_ns(exec, period, phase, cpu, cls, budget_policy,
				set_cpu_set)  );
	CALL( task_mode(LITMUS_RT_TASK) );
}

void make_litmus_decoder(struct thread_ctx *t_ctx)
{
	const lt_t budget = t_ctx->opts->budget * NSEC_PER_MSEC;
	const lt_t phase  = t_ctx->opts->phase * NSEC_PER_MSEC;

	make_litmus_task(budget, t_ctx->period, phase, 0, RT_CLASS_SOFT,
			NO_ENFORCEMENT, 0);
}

void make_litmus_display(struct thread_ctx *t_ctx)
{
	const lt_t phase  = t_ctx->opts->phase * NSEC_PER_MSEC;

	/* exec cost doesn't really matter for display */
	/* EDF-HSB does the enforcement for us, so NO_ENFORCEMENT is okay */

	make_litmus_task(1, t_ctx->period, phase, t_ctx->opts->cpu,
			RT_CLASS_HARD, NO_ENFORCEMENT, 1);
}

void wait_for_release(void)
{
	int err;
	err = wait_for_ts_release();
	if (err)
		die("Could not wait for TS release");
}

void become_regular_task(void)
{
	int ret;

	ret = task_mode(BACKGROUND_TASK);
	if (ret != 0)
		die("could not become regular task (huh?)");
}

#endif
