#include <stdio.h>
#include <errno.h>
#include <stdlib.h>

#include "inc/timing.h"

#if USE_POSIX_TIMERS

unsigned long long cpu_time(void)
{
	struct timespec ts;
	int err;
	err = clock_gettime(CLOCK_THREAD_CPUTIME_ID, &ts);
	if (err != 0)
	{
		perror("clock_gettime");
		exit(EXIT_FAILURE);
	}
	return (((unsigned long long)ts.tv_sec) * NSEC_PER_SEC + ts.tv_nsec);
}

void initialize_mono_time_t(mono_time_t* ts)
{
	ts->tv_sec = 0;
	ts->tv_nsec = 0;
}

#elif USE_APPLE_TIMERS

unsigned long long cpu_time(void)
{
	thread_port_t self;
	mach_msg_type_number_t count = THREAD_BASIC_INFO_COUNT;
	thread_basic_info_data_t info;
	kern_return_t err;

	self = mach_thread_self();

	err = thread_info(self, THREAD_BASIC_INFO,
			  (thread_info_t) &info, &count);

	if (err != KERN_SUCCESS || count != THREAD_BASIC_INFO_COUNT)
	{
		fprintf(stderr, "thread_info: err=%d count=%d\n", err, count);
		exit(EXIT_FAILURE);
	}

	/* Return user time only.  To also get the system time, add in
	 * info.system_time.seconds/microseconds.
	 * The casting of the info.user_time integers to unsigned long
	 * longs prior to multiplication is NECESSARY, otherwise overflow
	 * happens!
	 */
	return (((unsigned long long)info.user_time.seconds) * NSEC_PER_SEC +
			((unsigned long long)info.user_time.microseconds) * 1e3);
}

void initialize_mono_time_t(mono_time_t* t)
{
	*t = 0;
}

#else
#error no good clocks found
#endif /* USE_{POSIX or APPLE}_TIMERS */
